"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs");
const parser = require("@babel/parser");
const compiler = require("vue-template-compiler");
const traverse = require("@babel/traverse");
const scanResult_config_1 = require("./config/scanResult.config");
const technology_config_1 = require("./config/technology.config");
const chalk = require('chalk');
class baseScanService {
    async baseScan() {
        try {
            var dir = process.cwd();
            var dirArr = process.cwd().split('/');
            var projectName = dirArr[dirArr.length - 1];
            scanResult_config_1.default.allProjectsData[projectName] = {};
            scanResult_config_1.default.allProjectsData[projectName].dependence = {};
            let dependenceResult = await this.analysePackageJson(dir, projectName);
            if (dependenceResult) {
                await this.analyseFile(dir, projectName, 'suffix');
                console.log('分析文件完成...');
                scanResult_config_1.default.allProjectsData[projectName].technologyData &&
                    scanResult_config_1.default.allProjectsData[projectName].technologyData.componentsLibrary &&
                    console.log('分析组件库完成...');
                console.log(JSON.stringify(scanResult_config_1.default.allProjectsData[projectName].technologyData));
            }
            else {
                console.log(chalk.red('技术栈无数据，终止'));
            }
        }
        catch (err) {
            console.error(`${projectName}的技术栈扫描失败`);
            console.log(err);
        }
        return scanResult_config_1.default.allProjectsData[projectName];
    }
    async analyseFile(dir, projectName, condition) {
        const self = this;
        const readdirResult = fs.readdirSync(dir);
        if (!scanResult_config_1.default.allProjectsData[projectName]) {
            scanResult_config_1.default.allProjectsData[projectName] = {};
        }
        readdirResult.forEach((filename) => {
            const pathname = path.join(dir, filename);
            const stats = fs.statSync(pathname);
            if (stats.isDirectory()) {
                if (['node_modules', 'dist', '.git', '.vscode'].includes(filename)) {
                    return;
                }
                self.analyseFile(pathname, projectName, condition);
            }
            else if (stats.isFile()) {
                if (['.jpg', '.png', '.html'].includes(path.extname(pathname))) {
                    return;
                }
                if (condition == 'technology') {
                    if (filename == 'package.json') {
                        Object.assign(scanResult_config_1.default.allProjectsData[projectName].dependence || {}, this.jsonFormat(this.combineDependences(dir)) || {});
                    }
                }
                else if (condition == 'suffix') {
                    if (['.js', '.jsx', '.ts', '.tsx', '.vue'].includes(path.extname(pathname))) {
                        self.analyseComponentsLibrary(pathname, projectName);
                    }
                }
            }
        });
    }
    async analysePackageJson(dir, projectName) {
        let dependences = {};
        await this.analyseFile(dir, projectName, 'technology');
        dependences = scanResult_config_1.default.allProjectsData[projectName].dependence;
        if (dependences == {}) {
            return false;
        }
        const serchJson = [{}];
        for (const k in dependences) {
            if (Object.prototype.hasOwnProperty.call(dependences, k)) {
                serchJson.push({ name: k, version: dependences[k] });
            }
        }
        technology_config_1.default.technologyDataType.map(item => this.findDependence(projectName, dependences, item.name, item.reg));
        return true;
    }
    combineDependences(dir) {
        let allDependence = {};
        try {
            const readResult = JSON.parse(fs.readFileSync(path.join(dir, '/package.json'), 'utf-8'));
            allDependence = Object.assign(readResult.dependencies || {}, readResult.devDependencies || {});
        }
        catch (error) {
            console.log('读取合并package.json异常');
        }
        return allDependence;
    }
    jsonFormat(data) {
        for (const i in data) {
            if (Object.prototype.hasOwnProperty.call(data, i)) {
                data[i] = data[i].replace(/[^\d\.]+/g, '');
                const arr = data[i].split('.');
                data[i] = `${arr[0]}.${arr[1]}`;
            }
        }
        return data;
    }
    async findDependence(projectName, data, type, reg) {
        for (const i in data) {
            if (Object.prototype.hasOwnProperty.call(data, i)) {
                const saveData = Object.assign({}, scanResult_config_1.default.allProjectsData[projectName]);
                saveData.dependence = data;
                if (i.match(reg)) {
                    if (!saveData.technologyData) {
                        saveData.technologyData = JSON.parse(JSON.stringify(technology_config_1.default.technologyDataTemplate));
                    }
                    if (type === 'componentsLibrary') {
                        saveData.technologyData.componentsLibrary[i] = {
                            version: data[i],
                            useComponents: {},
                        };
                    }
                    else {
                        saveData.technologyData[type].name = i;
                        saveData.technologyData[type].version = data[i];
                    }
                }
                scanResult_config_1.default.allProjectsData[projectName] = saveData;
            }
        }
        return scanResult_config_1.default;
    }
    judgeFileType(pathname) {
        let readFileResult = '';
        readFileResult = fs.readFileSync(pathname, 'utf-8');
        if (path.extname(pathname) === '.vue') {
            readFileResult = compiler.parseComponent(readFileResult).script ? compiler.parseComponent(readFileResult).script.content : '';
        }
        return readFileResult;
    }
    transferToAst(readFileResult, pathname) {
        let astResult;
        try {
            astResult = parser.parse(readFileResult, {
                sourceType: 'module',
                plugins: [
                    'jsx',
                    'typescript',
                    'classProperties',
                    'classPrivateProperties', 'classPrivateMethods',
                    'decorators-legacy',
                ],
            });
        }
        catch (err) {
            console.log(`解析报错${pathname}${err}`);
        }
        return astResult;
    }
    analyseComponentsLibrary(pathname, projectName) {
        const resultContent = this.judgeFileType(pathname);
        let regRules = {};
        const astContext = resultContent ? this.transferToAst(resultContent, pathname) : '';
        technology_config_1.default.technologyDataType.map((item) => {
            if (item.name === 'componentsLibrary') {
                regRules = item.reg;
            }
            return regRules;
        });
        traverse.default(astContext, {
            ImportDeclaration: ({ node }) => {
                if (node.source.value.match(regRules)) {
                    let componentCon;
                    if (!scanResult_config_1.default.allProjectsData[projectName].technologyData.componentsLibrary[node.source.value]) {
                        scanResult_config_1.default.allProjectsData[projectName].technologyData.componentsLibrary[node.source.value] = { [node.source.value]: {} };
                    }
                    componentCon = scanResult_config_1.default.allProjectsData[projectName].technologyData.componentsLibrary[node.source.value]['useComponents'];
                    node.specifiers && node.specifiers.map((item, index) => {
                        if (item.imported && item.imported.name) {
                            const componentName = item.imported.name;
                            if (JSON.stringify(componentCon[componentName]) === '{}' || typeof componentCon[componentName] === 'undefined') {
                                componentCon[componentName] = { times: 1 };
                            }
                            else {
                                componentCon[componentName].times++;
                            }
                        }
                    });
                }
            }
        });
    }
}
exports.default = baseScanService;
