import * as path from 'path';
import * as fs from 'fs';
// import * as acorn from 'acorn';
import * as parser from '@babel/parser';
import * as compiler from 'vue-template-compiler';
// import * as compiler from '@vue/compiler-dom';
// import * as jsx from 'acorn-jsx';
import * as traverse from '@babel/traverse';
import scanResult from './config/scanResult.config';
import technology from './config/technology.config';
const chalk = require('chalk');


export default class baseScanService {
    /**
     * @description: 基础扫描器 遍历项目整体文件(目前只是简单扫描，规则后期补充)
     * @param {string} dir 当前扫描文件目录
     * @param {string} projectName 扫描项目名称
    */
    async baseScan() {

        try {

            var dir = process.cwd();
            var dirArr = process.cwd().split('/');
            var projectName = dirArr[dirArr.length-1];
            // 清空项目上次扫描结果
            scanResult.allProjectsData[projectName] = {};
            scanResult.allProjectsData[projectName].dependence = {}
            
            let dependenceResult = await this.analysePackageJson(dir, projectName);
            // 查询到技术栈依赖才进行下一步技术栈分析，否则终止技术栈扫描
            if( dependenceResult ){
                await this.analyseFile(dir, projectName, 'suffix');
                console.log('分析文件完成...')
                // 根据返回结果保存组件信息
                // console.log(scanResult.allProjectsData[projectName].technologyData)
                scanResult.allProjectsData[projectName].technologyData &&
                scanResult.allProjectsData[projectName].technologyData.componentsLibrary && 
                console.log('分析组件库完成...')
                console.log(JSON.stringify(scanResult.allProjectsData[projectName].technologyData))
            }
            else{
                console.log(chalk.red('技术栈无数据，终止'))
            }
            
        } catch (err) {
            console.error(`${projectName}的技术栈扫描失败`);
            console.log(err);
        }

        // cbk && cbk(scanResult.allProjectsData[projectName]);
        return scanResult.allProjectsData[projectName];
    }

    /**
     * @description: 根据文件后缀，统计文件类型
     * @param {type} dir 当前扫描文件目录
     * @param {type} projectName 扫描项目名称
     * @param {type} condition 查找条件: suffix(文件后缀)||technology(技术栈)
    */
     async analyseFile(dir:string, projectName:string, condition:any) {
        const self = this;
        const readdirResult = fs.readdirSync(dir);

        if (!scanResult.allProjectsData[projectName]) {
            scanResult.allProjectsData[projectName] = {};
        }

        readdirResult.forEach((filename) => {
            const pathname = path.join(dir, filename);
            const stats = fs.statSync(pathname);

            if (stats.isDirectory()) {
                // console.log('文件夹目录'+pathname)
                // 排除node_module和dist目录
                if (['node_modules', 'dist', '.git', '.vscode'].includes(filename)) {
                    return;
                }

                self.analyseFile(pathname, projectName, condition);
            } else if (stats.isFile()) {
                if (['.jpg', '.png', '.html'].includes(path.extname(pathname))) { // 排除图片和html文件
                    return;
                }
                if( condition == 'technology' ){
                    if( filename == 'package.json' ){

                        Object.assign(scanResult.allProjectsData[projectName].dependence || {}, this.jsonFormat(this.combineDependences(dir)) || {})

                    }
                }
                else if( condition == 'suffix' ){
                    if (['.js', '.jsx', '.ts', '.tsx', '.vue'].includes(path.extname(pathname))) {
                        // 解析组件库使用情况(前提是先把技术栈遍历完)
                        self.analyseComponentsLibrary(pathname, projectName);
                    }
                }
            }
        });
    }


    /**
     * @description: 统计项目技术栈 通过读取package.json文件
     * @param {string} dir 当前扫描文件目录
     * @param {string} projectName 扫描项目名称
    */
    async analysePackageJson(dir:string, projectName:string) {
        let dependences = {};
        // 遍历扫描所有package.json,存储到scanResult对象中
        await this.analyseFile(dir, projectName, 'technology');

        dependences = scanResult.allProjectsData[projectName].dependence;
        if( dependences == {} ){
            // 未扫描到依赖 直接返回false，不执行下面的代码
            return false;
        }


        const serchJson = [{}];

        // 记录本次扫描的name和version
        for (const k in dependences) {
            if (Object.prototype.hasOwnProperty.call(dependences, k)) {
                serchJson.push({ name: k, version: dependences[k] });
            }
        }
        // 触发技术栈分析
        technology.technologyDataType.map(item => this.findDependence(projectName, dependences, item.name, item.reg));
        return true;
    }

    /**
     * @description: 合并package.json中所有依赖
     * @param {type} dir 当前扫描文件目录
     * @return 返回所有依赖
     */
    combineDependences(dir:string) {
        let allDependence = {};
        try {
            const readResult = JSON.parse(fs.readFileSync(path.join(dir, '/package.json'), 'utf-8'));
            allDependence = Object.assign(readResult.dependencies || {}, readResult.devDependencies || {});

        } catch (error) {
           console.log('读取合并package.json异常') 
        }


        // 合并扫描项目package.json中依赖
        return allDependence;
    }

    jsonFormat(data:any) {
        for (const i in data) {
            if (Object.prototype.hasOwnProperty.call(data, i)) {
                data[i] = data[i].replace(/[^\d\.]+/g, '');
                const arr = data[i].split('.');

                data[i] = `${arr[0]}.${arr[1]}`;
            }
        }

        return data;
    }

   

    /**
     * @description: 匹配依赖，统计技术栈及版本号
     * @param {type} projectName 当前扫描项目名称
     * @param {type} data package.json依赖集合
     * @param {type} type 所属技术栈分类
     * @param {type} reg 依赖名称与技术栈名称匹配规则
     * @return 返回扫描结果
    */
    async findDependence(projectName:string, data:any, type:any, reg:any) {
        for (const i in data) {
            if (Object.prototype.hasOwnProperty.call(data, i)) {
                const saveData = { ...scanResult.allProjectsData[projectName] };
                // 存储package.json
                saveData.dependence = data;
                if (i.match(reg)) {
                    if (!saveData.technologyData) {
                        saveData.technologyData = JSON.parse(JSON.stringify(technology.technologyDataTemplate));
                    }

                    // 特殊处理componentsLibrary组件库，一个项目中可能存在多个组件库
                    if (type === 'componentsLibrary') {
                        saveData.technologyData.componentsLibrary[i] = {
                            version: data[i],
                            useComponents: {},
                        };
                    } else {
                        saveData.technologyData[type].name = i;
                        saveData.technologyData[type].version = data[i];
                    }
                }

                scanResult.allProjectsData[projectName] = saveData;

            }
        }

        return scanResult;
    }

    /**
     * @description: 判断文件类型
     * @description:
     * @description: vue文件的parser将内容转成sfc描述，script标签中的内容被整体解析成了字符串,需要提取script内容让babel-parser解析成ast
     * @param {type} pathname 文件路径
     * @return {string} readFileResult 返回需要解析的内容的
    */
    judgeFileType(pathname:string) {
        let readFileResult = '';

        readFileResult = fs.readFileSync(pathname, 'utf-8'); // 读取文件内容

        if (path.extname(pathname) === '.vue') {
            readFileResult = compiler.parseComponent(readFileResult).script ? compiler.parseComponent(readFileResult).script.content : '';
        }

        return readFileResult;
    }

    /**
     * @description: 解析文件内容(字符串 to AST)
     * @param {type} pathname 文件路径
     * @return 返回ast语法树
    */
    transferToAst(readFileResult:any, pathname:string) {
        let astResult;

        // 根据指定文件类型，调用相应的ast解析器，解析成ast再进行内容分析
        try {
            astResult = parser.parse(readFileResult, {
                sourceType: 'module',
                plugins: [
                    'jsx',
                    'typescript',
                    'classProperties',
                    'classPrivateProperties', 'classPrivateMethods',
                    'decorators-legacy',
                ],
            });
        } catch (err) {
            console.log(`解析报错${pathname}${err}`);
        }
        return astResult;
    }

    /**
     * @description 解析组件库使用情况
     */
    analyseComponentsLibrary(pathname:string, projectName:string) {
        const resultContent = this.judgeFileType(pathname);
        let regRules:any = {};

        const astContext = resultContent ? this.transferToAst(resultContent, pathname) : '';

        // 组件库匹配规则
        technology.technologyDataType.map((item) => {
            if (item.name === 'componentsLibrary') {
                regRules = item.reg;
            }

            return regRules;
        });
                    
        traverse.default(astContext, {
            // 查找import导入
            ImportDeclaration: ({ node }) => {
                if (node.source.value.match(regRules)) {
                    // console.log(scanResult.allProjectsData[projectName].technologyData.componentsLibrary[node.source.value])
                    let componentCon;
                    if( !scanResult.allProjectsData[projectName].technologyData.componentsLibrary[node.source.value] ){
                        scanResult.allProjectsData[projectName].technologyData.componentsLibrary[node.source.value] = {[node.source.value]:{}}
                    }
                    componentCon = scanResult.allProjectsData[projectName].technologyData.componentsLibrary[node.source.value]['useComponents'];

                    // 组件库内具体组件的使用情况
                    // import{Toast,Drawer} from 'cheui-react'
                    node.specifiers && node.specifiers.map((item:any,index)=>{
                        if( item.imported&&item.imported.name ){
                            const componentName = item.imported.name;
                            if (JSON.stringify(componentCon[componentName]) === '{}' || typeof componentCon[componentName] === 'undefined') {
                                componentCon[componentName] = { times: 1 };
                            } else {
                                componentCon[componentName].times++;
                            }
                        }
                    })

                }
            }
        });                  
    }
}


